/*
 * macros.h
 *
 * Created: 21/03/2014 10:15:46
 *  Author: marcadella
 */


// Macros definitions

#ifndef MACROS_H
#define MACROS_H

#include "projectConfig.h"

// Return values
#define FPASS	1
#define FFAIL	0

// Commodities

#define INT0			(AVR32_INTC_INT0)
#define INT1			(AVR32_INTC_INT1)
#define INT2			(AVR32_INTC_INT2)
#define INT3			(AVR32_INTC_INT3)


// Priorities

#define PRIORITY_MIN		0
#define PRIORITY_LOW		2
#define PRIORITY_MID		5
#define PRIORITY_HIGH		8
#define PRIORITY_MAX		10


// Constants definitions
#define FCPU_HZ 12000000
#define FPBA_HZ 12000000

// Macros for the LEDs
#if USE_LEDS == USE
	#define m_gpio_toggle_led(led)		gpio_toggle_pin(led)
	#define m_gpio_set_led_high(led)	gpio_set_pin_high(led)
	#define m_gpio_set_led_low(led)		gpio_set_pin_low(led)
#else
	#define m_gpio_toggle_led(led)
	#define m_gpio_set_led_high(led)
	#define m_gpio_set_led_low(led)
#endif


// Macro for starting FreeRTOS scheduler
#define mvTaskStartScheduler()																\
		gplp_set(RTOS_KERNEL);																\
		vTaskStartScheduler();																\
		gplp_clear(RTOS_KERNEL)


// Macro for initializing a task
#define mvTaskCreate(cond, a, b, c, d, e, f, fnctName)										\
		if (cond == USE)																	\
		{																					\
			if (a ## _check())																\
			{																				\
				if (xTaskCreate(a, (const signed char * const) b, c, d, e, f) == pdPASS)	\
					vPrintStrS(USE_VERBOSE, #b " task created\n");								\
				else																		\
					vErrorNotif(#b " task failed initialization.\n", fnctName);				\
			}																				\
			else																			\
			{																				\
				gplp_set(INITIALIZATION_ERROR);												\
				vErrorNotif(#b " task not created because check faild.\n", fnctName);		\
			}																				\
		}
		

// Macro for initializing a mutex
#define mxSemaphoreCreateMutex(cond, sem, fnctName)											\
		if (cond == USE)																	\
		{																					\
			sem = xSemaphoreCreateMutex();													\
			if (sem != NULL)																\
				vPrintStrS(USE_VERBOSE, #sem " created\n");										\
			else																			\
			{																				\
				gplp_set(INITIALIZATION_ERROR);												\
				vErrorNotif(#sem " failed initialization.\n", fnctName);					\
			}																				\
		}
		

// Macro for initializing a queue
#define mvQueueCreate(cond, queue, b, c, fnctName)											\
		if (cond == USE)																	\
		{																					\
			queue = xQueueCreate(b, c);														\
			if (queue != NULL)																\
				vPrintStrS(USE_VERBOSE, #queue " created\n");									\
			else																			\
			{																				\
				gplp_set(INITIALIZATION_ERROR);												\
				vErrorNotif(#queue " failed initialization.\n", fnctName);					\
			}																				\
		}


// Macro for initializing a binary semaphore
#define mvSemaphoreCreateBinary(cond, sem, giveFromStart, fnctName)							\
		if (cond == USE)																	\
		{																					\
			vSemaphoreCreateBinary(sem);													\
			if(sem != NULL)																	\
			{																				\
				vPrintStrS(USE_VERBOSE, #sem " initialized.\n");								\
				if (!giveFromStart)															\
					xSemaphoreTake(sem, 0);													\
			}																				\
			else																			\
			{																				\
				gplp_set(INITIALIZATION_ERROR);												\
				vErrorNotif(#sem " failed initialization.\n", fnctName);					\
			}																				\
		}
		

// Macro for checking the initialization of the required elements fo a task to run properly
#define mCheck(a, b, c, d, e, f, g, h, i, j, fnctName)								\
		short check = 1;															\
		if ((void*)a == NULL)														\
		{																			\
			check = 0;																\
			vErrorNotif(#a " = NULL.\n", fnctName);									\
		}																			\
		if ((void*)b == NULL)														\
		{																			\
			check = 0;																\
			vErrorNotif(#b " = NULL.\n", fnctName);									\
		}																			\
		if ((void*)c == NULL)														\
		{																			\
			check = 0;																\
			vErrorNotif(#c " = NULL.\n", fnctName);									\
		}																			\
		if ((void*)d == NULL)														\
		{																			\
			check = 0;																\
			vErrorNotif(#d " = NULL.\n", fnctName);									\
		}																			\
		if ((void*)e == NULL)														\
		{																			\
			check = 0;																\
			vErrorNotif(#e " = NULL.\n", fnctName);									\
		}																			\
		if ((void*)f == NULL)														\
		{																			\
			check = 0;																\
			vErrorNotif(#f " = NULL.\n", fnctName);									\
		}																			\
		if ((void*)g == NULL)														\
		{																			\
			check = 0;																\
			vErrorNotif(#g " = NULL.\n", fnctName);									\
		}																			\
		if ((void*)h == NULL)														\
		{																			\
			check = 0;																\
			vErrorNotif(#h " = NULL.\n", fnctName);									\
		}																			\
		if ((void*)i == NULL)														\
		{																			\
			check = 0;																\
			vErrorNotif(#i " = NULL.\n", fnctName);									\
		}																			\
		if ((void*)j == NULL)														\
		{																			\
			check = 0;																\
			vErrorNotif(#j " = NULL.\n", fnctName);									\
		}																			\
		return check
		

// Macro for getting a randum number between min and max
#define mrand(mmin, mmax)  ((rand() % (mmax - mmin + 1)) + mmin)


// Writes a char on the DBG usart line
#define m_usart_putchar(a, b, fnctName)												\
		if (USART_SUCCESS != usart_putchar(a, b))									\
			vErrorNotif("Write on USART failed\n", fnctName)
		
#if USE_DBG == USE
	#define mNotifyfromBoot(s)	NotifyfromBoot(s)
#else
	#define mNotifyfromBoot(s)
#endif


#endif // MACROS_H