/**
 * \main
 *
 * \brief NUTS OBC.
 *
 */

#include <asf.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <csp/csp.h>
#include <csp/csp_nuts.h>
#include <csp/arch/csp_thread.h>

#include "services/event_man.h"
#include "services/timekeeper.h"
#include "services/logger.h"
#include "services/housekeeper.h"
#include "services/helper_tasks.h"

uint8_t obc_tasks_init(void);

int main(void){
	/* Initilize the SAM system */
	sysclk_init();
	board_init();
	
	/* Initialize the console uart */
	//configure_console();
	
	/* Initialize the debug tasks (led_blink mm.) */
	helper_tasks_init();
	
	/* Configure and initialize CSP */
	csp_nuts_init();
	
	/* Initialize the OBC tasks */
	obc_tasks_init();

	/* Start the scheduler. */
	vTaskStartScheduler();

	/* Will only get here if there was insufficient memory to create the idle task. */
	return 0;
}

/**
* Initialize OBC threads.
*/
uint8_t obc_tasks_init(void){
	/* event_man */
	printf("Starting event manager task\r\n");
	csp_thread_handle_t handle_event_man;
	csp_thread_create(task_event_man, "events", 1000, NULL, 0, &handle_event_man);

	/* timekeeper */
	printf("Starting timekeeper task\r\n");
	csp_thread_handle_t handle_timekeeper;
	csp_thread_create(task_timekeeper, "timek", 1000, NULL, 0, &handle_timekeeper);
	
	/* logger */
	printf("Starting memory manager task\r\n");
	csp_thread_handle_t handle_logger;
	csp_thread_create(task_logger, "memman", 1000, NULL, 0, &handle_logger);
	
	/* housekeeper */
	printf("Starting housekeeper task\r\n");
	csp_thread_handle_t handle_housekeeper;
	csp_thread_create(task_housekeeper, "housek", 1000, NULL, 0, &handle_housekeeper);
}
