/*
 * tickless_idle.c
 *
 * Created: 18.03.2016 15:30:09
 *  Author: Normann
 */ 
/* ASF includes. */
#include "sysclk.h"
#include <asf.h>

/* FreeRTOS includes. */
#include "FreeRTOS.h"
#include "task.h"

#include "middleware/tickless_idle.h"
#include "conf_sleepmgr.h"

void tickless_idle_configure(void){
	/* Ensure the 32KHz oscillator is enabled. */
	//TODO::
	
	uint32_t ul_previous_time;

	/* Configure RTT to use RTC 32KHz clock */
	//rtt_sel_source(RTT, false);	//32KHz source
	//rtt_sel_source(RTT, true);		//1Hz source
	rtt_init(RTT, CONF_RTT_PRESCALER_VAL);

	ul_previous_time = rtt_read_timer_value(RTT);
	while (ul_previous_time == rtt_read_timer_value(RTT));

	/* Enable RTT interrupt */
	NVIC_DisableIRQ(RTT_IRQn);
	NVIC_ClearPendingIRQ(RTT_IRQn);
	NVIC_SetPriority(RTT_IRQn, configLIBRARY_LOWEST_INTERRUPT_PRIORITY);
	NVIC_EnableIRQ(RTT_IRQn);
	rtt_enable_interrupt(RTT, RTT_MR_ALMIEN);
	
	/* Set wakeup source to rtt_alarm */
	pmc_set_fast_startup_input(PMC_FSMR_RTTAL);
	supc_set_wakeup_mode(SUPC, SUPC_WUMR_RTTEN_ENABLE);
	
	/* Initialize the sleep manager */
	sleepmgr_init();
		
	/* Ensure the RTT can bring the CPU out of sleep mode. */
	sleepmgr_lock_mode(CONF_SLEEP_MODE);
}

void RTT_Handler( void ){
	/* The CPU woke because of a tick. */
	ulTickFlag = pdTRUE;
	
	/* Get RTT status */
	uint32_t ul_status;
	ul_status = rtt_get_status(RTT); //flag is cleared when read

	/* Alarm */
	if ((ul_status & RTT_SR_ALMS) == RTT_SR_ALMS) {
	}
}

void vPortSuppressTicksAndSleep( portTickType xExpectedIdleTime ){
unsigned long ulReloadValue, ulCompleteTickPeriods, ulCompletedSysTickIncrements, ulTimeSeconds, ulTimeMilliSeconds;
unsigned long ulCalValue_before, ulCalValue_after;
portTickType xModifiableIdleTime;
	if(xExpectedIdleTime < CONF_MINIMUM_ALOWABLE_SLEEP_TIME){
		return;
	}
	
// 	/* Make sure the SysTick reload value does not overflow the counter. */	
// 	if( xExpectedIdleTime > xMaximumPossibleSuppressedTicks )	{
// 		xExpectedIdleTime = xMaximumPossibleSuppressedTicks;
// 	}	
	
	//ulTimeSeconds = xExpectedIdleTime / configTICK_RATE_HZ;
	ulTimeMilliSeconds = (xExpectedIdleTime*1000) / configTICK_RATE_HZ;
	/* if sleep period less than 1s, do nothing and return back */
	if(ulTimeMilliSeconds == 0)
		return;
	if(ulTimeMilliSeconds > MAXIMUM_POSSIBLE_SLEEP_TIME) {
		ulTimeMilliSeconds = MAXIMUM_POSSIBLE_SLEEP_TIME;
	}
	ulTickFlag = pdFALSE;
	
	/* Calculate the reload value required to wait xExpectedIdleTime
	tick periods.  -1 is used because this code will execute part way
	through one of the tick periods, and the fraction of a tick period is
	accounted for later. */
/*
	ulReloadValue = ( ulTimerReloadValueForOneTick * ( xExpectedIdleTime - 1UL ) );
	if( ulReloadValue > ulStoppedTimerCompensation )
	{
		ulReloadValue -= ulStoppedTimerCompensation;
	}
*/
	/* Stop the SysTick momentarily.  The time the SysTick is stopped for
	is accounted for as best it can be, but using the tickless mode will
	inevitably result in some tiny drift of the time maintained by the
	kernel with respect to calendar time. */
/*		portNVIC_SYSTICK_CTRL_REG = portNVIC_SYSTICK_CLK_BIT | portNVIC_SYSTICK_INT_BIT;*/

	/* Adjust the reload value to take into account that the current time
	slice is already partially complete. */
/*		ulReloadValue += ( portNVIC_SYSTICK_LOAD_REG - ( portNVIC_SYSTICK_LOAD_REG - portNVIC_SYSTICK_CURRENT_VALUE_REG ) );*/

	/* Enter a critical section but don't use the taskENTER_CRITICAL()
	method as that will mask interrupts that should exit sleep mode. */
	__asm volatile( "cpsid i" );

	/* If a context switch is pending or a task is waiting for the scheduler
	to be unsuspended then abandon the low power entry. */
	if( eTaskConfirmSleepModeStatus() == eAbortSleep )
	{
		/* Restart SysTick. */
/*			portNVIC_SYSTICK_CTRL_REG = portNVIC_SYSTICK_CLK_BIT | portNVIC_SYSTICK_INT_BIT | portNVIC_SYSTICK_ENABLE_BIT;*/

		/* Re-enable interrupts - see comments above the cpsid instruction()
		above. */
		__asm volatile( "cpsie i" );
	}
	else
	{
		/* Set the new reload value. */
/*			portNVIC_SYSTICK_LOAD_REG = ulReloadValue;*/

		/* Clear the SysTick count flag and set the count value back to
		zero. */
/*			portNVIC_SYSTICK_CURRENT_VALUE_REG = 0UL;*/
		/* Restart SysTick. */
/*			portNVIC_SYSTICK_CTRL_REG = portNVIC_SYSTICK_CLK_BIT | portNVIC_SYSTICK_INT_BIT | portNVIC_SYSTICK_ENABLE_BIT;*/
			
		/* Store the user alarm value */
		//TODO:: ulUserAlarmValue = ast_read_alarm0_value(AST);
		
			
		/* Set alarm register */			
		//rtt_init(RTT, CONF_RTT_PRESCALER_VAL);
		rtt_write_alarm_time(RTT, rtt_read_timer_value(RTT) + (ulTimeMilliSeconds*RTT_TICKS_PER_MILLI_SEC));
			
// 		/* Read current calendar value */
// 		xCalendar = ast_read_calendar_value(AST);
// 			
// 		if(xCalendar.field < ulUserAlarmValue) {
// 			/* if user alarm occurs before the mcu woken up time, 
// 			use the user alarm to wake up mcu */
// 			if(ast_read_alarm0_value(AST) > ulUserAlarmValue) {
// 				ast_write_alarm0_value(AST, ulUserAlarmValue);
// 			}
// 		}
// 		if(xCalendar.field >= ast_read_alarm0_value(AST)) {
// 			/* Don't need to restore the user alarm value */
// 			return;
// 		}
		if(ulTickFlag == true) {
			/* AST Alarm interrupt has occurred, not 
			need to restore the user alarm value */
			return;
		}
		/* Sleep until something happens.  configPRE_SLEEP_PROCESSING() can
		set its parameter to 0 to indicate that its implementation contains
		its own wait for interrupt or wait for event instruction, and so wfi
		should not be executed again.  However, the original expected idle
		time variable must remain unmodified, so a copy is taken. */
		xModifiableIdleTime = xExpectedIdleTime;
		vPreSleepProcessing( xModifiableIdleTime );
		if( xModifiableIdleTime > 0 )
		{
			/* Sleep until something happens. */
			sleepmgr_enter_sleep();
/*				
			__asm volatile( "wfi" );
			__asm volatile( "dsb" );
			__asm volatile( "isb" );
*/
		}
		vPostSleepProcessing( xExpectedIdleTime );

		/* Stop SysTick.  Again, the time the SysTick is stopped for is
		accounted for as best it can be, but using the tickless mode will
		inevitably result in some tiny drift of the time maintained by the
		kernel with respect to calendar time. */
/*			portNVIC_SYSTICK_CTRL_REG = portNVIC_SYSTICK_CLK_BIT | portNVIC_SYSTICK_INT_BIT;*/
			
		if(ulTickFlag != pdFALSE) {
			ulCompleteTickPeriods = (ulTimeMilliSeconds * configTICK_RATE_HZ)/1000;
			/* Restore user alarm value */
			//TODO:: ast_write_alarm0_value(AST, ulUserAlarmValue);
		}
		else {
			/* Something other than the AST interrupt ended the sleep.*/
			//only calculate minutes and seconds, on the premise of MAXIMUM_POSSIBLE_SLEEP_TIME.
// 			ulCalValue_before = xCalendar.FIELD.min * 60 + xCalendar.FIELD.sec;
// 			xCalendar = ast_read_calendar_value(AST);
// 			if((xCalendar.FIELD.min == 0) && (ulCalValue_before > MAXIMUM_SECONDS_IN_MINUTE)) { //minute field overflow, hour increased by 1
// 				ulCalValue_after = MAXIMUM_MINUTES_IN_HOUR * MAXIMUM_SECONDS_IN_MINUTE + xCalendar.FIELD.sec;
// 			}
// 			else {
// 				ulCalValue_after = xCalendar.FIELD.min * 60 + xCalendar.FIELD.sec;
// 			}
			ulCompleteTickPeriods = ((rtt_read_timer_value(RTT)/RTT_TICKS_PER_MILLI_SEC) * configTICK_RATE_HZ)/1000;
			/* Restore user alarm value */
			//ast_write_alarm0_value(AST,ulUserAlarmValue);
		}
		/* Restart SysTick so it runs from portNVIC_SYSTICK_LOAD_REG
		again, then set portNVIC_SYSTICK_LOAD_REG back to its standard
		value. */
/*			
		portNVIC_SYSTICK_CURRENT_VALUE_REG = 0UL;
		portNVIC_SYSTICK_CTRL_REG = portNVIC_SYSTICK_CLK_BIT | portNVIC_SYSTICK_INT_BIT | portNVIC_SYSTICK_ENABLE_BIT;
*/
		vTaskStepTick( ulCompleteTickPeriods );
			
		/* Re-enable interrupts - see comments above the cpsid instruction()
		above. */
		__asm volatile( "cpsie i" );
	}
}

void vPreSleepProcessing( unsigned long ulExpectedIdleTime )
{
	/* Called by the kernel before it places the MCU into a sleep mode because
	configPRE_SLEEP_PROCESSING() is #defined to vPreSleepProcessing().

	NOTE:  Additional actions can be taken here to get the power consumption
	even lower.  For example, peripherals can be turned	off here, and then back
	on again in the post sleep processing function.  For maximum power saving
	ensure all unused pins are in their lowest power state. */

	/* Avoid compiler warnings about the unused parameter. */
	( void ) ulExpectedIdleTime;
}
/*-----------------------------------------------------------*/

void vPostSleepProcessing( unsigned long ulExpectedIdleTime )
{
	/* Called by the kernel when the MCU exits a sleep mode because
	configPOST_SLEEP_PROCESSING is #defined to vPostSleepProcessing(). */

	/* Avoid compiler warnings about the unused parameter. */
	( void ) ulExpectedIdleTime;
}